@@ -20,16 +20,26 @@ def request_factory(environ):
             return '<IHTTPRequest %s>' % urllib.urlencode(environ)
 
     IHTTPRequest = Interface("IHTTPRequest", (IRequest,))
-        
+
     ifaces = []
-    for name, value in environ.items():
-        key = (name, str(value).lower())
+    for name, token in environ.items():
+        # if token is a string, then use a standard comparison method
+        # which checks if the request has a value which appears in the
+        # lowercased match string
+        if isinstance(token, (str, unicode)):
+            normalized_value = str(token).lower()
+            method = lambda other: normalized_value in other
+        # else, assume token is a truth function
+        else:
+            method = token
+
+        key = name, method
         iface = global_registry.get(key)
         if iface is None:
             class IMarker(interface.Interface):
                 pass
             IMarker.__module__ = interfaces
-            IMarker.__name__ = "I"+(name+value).encode('hex_codec')
+            IMarker.__name__ = ("I%s%d" % (name, id(token))).encode('hex_codec')
             iface = global_registry[key] = IMarker
 
         ifaces.append(iface)
@@ -50,7 +60,7 @@ def request_factory(environ):
 
 # add adapter hook to adapt dicts to the ``IRequest`` interface
 def adapt_dict_to_request(iface, ob):
-    if iface is IRequest:
+    if iface.isOrExtends(IRequest):
         return request_factory(ob)
     
 adapter_hooks.append(adapt_dict_to_request)
@@ -64,7 +74,8 @@ def new_request_handler(new_request_event):
         value1 = str(value1).lower()
         for (name2, value2), iface in global_registry.items():
             name2 = name2.lower()
-            if name1 == name2 and value2 in value1:
+            
+            if name1 == name2 and value2(value1):
                 ifaces.append(iface)
 
     ifaces.sort(key=operator.attrgetter('__name__'))
